/* 
** A simple fire routine written in DJGPP C with no Assembly
**
** Prog. by Ranjith K Thamizhan ( ranjithk@crosswinds.net )
** Date: 8/4/2000
**
** Please send me a mail, if you find this code useful.
*/


#include<dos.h>
#include<sys/nearptr.h>

#ifndef SAVE_NO_SPACE
/* 
** Creating dummy versions of these three functions in DJGPP 
** will prevent some EXE bloat. ( UPX/DJP saves even more ) 
*/

char **__crt0_glob_function (char *arg){ return 0; }
void   __crt0_load_environment_file(char *_app_name){ return; }
void   __crt0_setup_arguments(void){ return; }
#endif

typedef unsigned char byte;

void setrgb( byte c, byte r, byte g, byte b )
{
  outportb(0x3C8,c);   
  outportb(0x3C9,r);   
  outportb(0x3C9,g);   
  outportb(0x3C9,b);   
}

void set_palette(int type)
{
  int x=0;

  if( type==1 )
    for (x=1; x<=32; x++) {
        setrgb(x,   x*2-1, 0,     0    );       /* red */
        setrgb(x+32, 63,   x*2-1,  0   );
        setrgb(x+64, 63,   63,   x*2-1);
        setrgb(x+96, 63,   63,    63);
    }
  else if( type==2 )
    for (x=1; x<=32; x++) {
        setrgb(x,   0, 0,     x*2-1    );       /* blue */
        setrgb(x+32, x*2-1, 0,   63    );
        setrgb(x+64, 63,   x*2-1,   63);
        setrgb(x+96, 63,   63,    63);
    }
  else
    for (x=1; x<=32; x++) {
        setrgb(x,   0, x*2-1,     0    );       /* green */
        setrgb(x+32, 0, 63,   x*2-1    );
        setrgb(x+64, x*2-1,   63,   63);
        setrgb(x+96, 63,   63,    63);
    }

}

int main()
{
  union REGS s;

  byte *scr;
  byte *vrscr;
  byte *t;
  byte *val;
  int   key;

  vrscr = (char*) calloc(320*200,1);

  if( vrscr == 0 )
  {
    printf("Not enough memory\n");
    return 0;
  }

  printf("This simple fire effect was written entirely in DJGPP C compiler\n"
         "with no assembly language.\n\n"
         "Try pressing 'r','g' or 'b' for different colored flames.\n\n"
	 "prog. by Ranjith K ( ranjithk@crosswinds.net ) \n\n"
         "press a key to continue\n");

  getch();

  /* Set graphics mode */

  s.w.ax = 0x13;
  int86(0x10, &s,&s);

  /* start with the real red */

  set_palette(1);

  /* Enable full memory access like Watcom C / TCC */

  __djgpp_nearptr_enable();

  scr = (byte*)0xA0000 + __djgpp_conventional_base;

  while( 1 )
  { 
    val = vrscr+320*200;

/* This traingle part is optional. Included to draw a simple 
** dotted triangle at the bottom */

#ifdef DRAW_TRIANGLE
    *(vrscr+145*320+160) = 140;
    *(vrscr+145*320+140) = 140;
    *(vrscr+145*320+180) = 140;
    *(vrscr+145*320+150) = 140;
    *(vrscr+145*320+170) = 140;

    *(vrscr+152*320+145) = 140;
    *(vrscr+152*320+175) = 140;

    *(vrscr+160*320+150) = 140;
    *(vrscr+160*320+170) = 140;

    *(vrscr+167*320+155) = 140;
    *(vrscr+167*320+165) = 140;

    *(vrscr+175*320+160) = 140;
#endif


    /* fill the bottom line with random values */

    for( t=val-640; t<val; t++ )
    {
        *t = (rand()%2)==0 ? 120 : 0; 
    }

    val = vrscr+320*200-320;


    /* Heart of the fire routine, calculate pixel values 
    **--------------------------------------------------
    */

    for( t=vrscr+320*100; t<val; t++ )
    {
        *(t-320) = (*t + *(t+1) + *(t-1) + *(t+320) + 
	              *(t+321) + *(t+319)) / 6; 


        /* The fire needs to fade at the top */

        if( *(t-320) > 0 )
            *(t-320) = *(t-320) - 1;

    }

    /*
    **--------------------------------------------------
    */

    /* blit buffer to screen */

    memcpy( scr+(320*4), vrscr, 320*197 );


    /* switch palette if necessary */

    if( kbhit() )
        if( (key = getch()) == 27 )
            break;
        else if( key == 'r' )
            set_palette(1);
        else if( key == 'g' )
            set_palette(3);
        else if( key == 'b' )
            set_palette(2);
  }

  free(vrscr);

  
  __djgpp_nearptr_disable();

  /* Set text mode */ 

  s.w.ax = 0x3;
  int86(0x10, &s,&s);
  
  return 0;
}
