/* 
** Simple snow effect. Change the col1/col2 variables to reflect
** changed color of the snow. Compile with Watcom C. Porting to 
** other C compilers shouldn't be a problem
**
** Note: This snow effect uses two colors for better effect. 
** A single color could be used for simplicity
**
** Prog. by Ranjith K Thamizhan ( ranjithk@crosswinds.net )
** Date: 7/29/00
**
** I guess the source code is simple enough to understand
** how it works.
**
** Any way...
** ----------
** We fill the top line of the screen randomly with a one of two 
** colors and we make sure we have fewer dots filled on top 
** otherwise this will result in a messy screen of dots. Then 
** begin from the line just before the bottom line and go upto 
** the top. For every pixel we come across, we check if it's colored, 
** if so we move it just below one pixel and one pixel to the left or 
** right randomly. If the left or right pixel is colored try moving 
** it to the right or left respectively. If even that one is colored 
** try moving it just down one pixel. If the pixel below is also 
** filled simply leave it as it is. This will create an effect of 
** the snow deposits growing on top of each other.
**
** To simulate a more realistic snow effect we need to use many small 
** sized bitmaps to hold pictures of tiny lumps of snow of varying size. 
** Note that this thing is not implemented here.
** 
** Please send me a mail, if you find this code useful.
*/


#include<i86.h>

int main()
{
    unsigned char *scr = (unsigned char*)0xA0000;    
    unsigned char *vscr;
    union REGS	  s;
    int           t;
    const int     col1=78, col2=77;
    int           script_frames = 100;


    /* graphics video mode */

    s.w.ax = 0x13;
    int386(0x10,&s,&s);
    
    vscr = (unsigned char*) malloc( 320*200 );

    if( !vscr )
    {
	printf("Not enough memory");
	return(0);
    }

    memset(vscr, 0, 320*200 );

    while( !kbhit() )
    {

    /* The top line */

    for(t=0; t<320; t++)
        *(vscr+t)= (rand()%55)==54 ? ((rand()%55)==54 ? col1: col2 ): 0;
    
    for(t=320*200-322; t >= 0; t-- )
    {

	/* If it's edge of the screen, skip processing and blank pixel */

        if( t%320==0 || t%320==1 )
        {
            *(vscr+t)=0;
            continue;
        }
	
	/* colored ? */

        if( *(vscr+t) )
        {
	    /* get a random side */

            if( rand()%2 )
            {
                /* left side */

                if( *(vscr+t+319) )
                {
                    if( *(vscr+t+321) )
                    {
                        if( *(vscr+t+320)==0 )
                        {
                            *(vscr+t+320) = rand()%2 ? col1: col2;
                            *(vscr+t)=0;
                        }
                    }
		    else
                    {
                        *(vscr+t+319) = rand()%2 ? col1: col2;
                        *(vscr+t) = 0;
                    }

                }
                else
                {
                    *(vscr+t+319) = rand()%2 ? col1: col2;
                    *(vscr+t) = 0;
                }
            }
            else
            {
                /* right side */

                if( *(vscr+t+321) )
                {
                    if( *(vscr+t+319) )
                    {
                        if( *(vscr+t+320)==0 )
                        {
                            *(vscr+t+320) = rand()%2 ? col1: col2;
                            *(vscr+t)=0;
                        }
                    }
                    else
                    {
                        *(vscr+t+319) = rand()%2 ? col1: col2;
                        *(vscr+t) = 0;
                    }
                }
                else
                {
                    *(vscr+t+321) = rand()%2 ? col1: col2;
                    *(vscr+t) = 0;
                }

            }
        }

    }

    /* For dots, not so great speed improvement, yet... */

    memcpy(scr,vscr,320*200);

    }

    getch();

    /* switch back to text mode */

    s.w.ax = 0x3;
    int386(0x10, &s,&s);

    free(vscr);

    return(0);
}